/********************************** (C) COPYRIGHT *******************************
* File Name          : Main.c
* Author             : kingsley @wch
* Version            : V1.1
* Date               : 2021/8/23
* Description 		 : iic implementation on ch577/578/579/
*******************************************************************************/


#include "ch57x_soft_i2c.h"
#include "CH57x_common.h"

#include "oled.h"



//PB12-SCL
//PB13-SDA
#define PIN_SDA                 GPIO_Pin_10
#define PIN_SCL                 GPIO_Pin_11
#define GPIO_SetBits(x,y)       (R32_PB_DIR    &= ~(y))
#define GPIO_ResetBits(x,y)     (R32_PB_DIR  |= (y))
#define I2C_READ_SDA()          (GPIOB_ReadPortPin(PIN_SDA)?1:0)

//about 100K
#define DELAY_TIMES     (0)

#define Delay_us(x) DelayUs(x)





//below is not needed in this implementation
#define I2C_PORT 1
#define SDA_OUT()
#define SDA_IN()



void ch57x_soft_i2c_init(void) {
    GPIOB_ModeCfg(PIN_SCL, GPIO_ModeOut_PP_20mA);
    GPIOB_ModeCfg(PIN_SDA, GPIO_ModeOut_PP_20mA);
    GPIOB_SetBits(PIN_SCL);
    GPIOB_SetBits(PIN_SDA);
}


void ch57x_soft_i2c_start(void) {
    SDA_OUT();
    GPIO_SetBits(I2C_PORT,PIN_SDA);  //SDA=1
    Delay_us(DELAY_TIMES);
    GPIO_SetBits(I2C_PORT,PIN_SCL);  //SCL=1
    Delay_us(DELAY_TIMES);
    GPIO_ResetBits(I2C_PORT,PIN_SDA);//SDA=0
    Delay_us(DELAY_TIMES);
    GPIO_ResetBits(I2C_PORT,PIN_SCL);//SCL=0
    Delay_us(DELAY_TIMES);
}

void ch57x_soft_i2c_stop(void) {
    SDA_OUT();
    GPIO_ResetBits(I2C_PORT,PIN_SDA);
    Delay_us(DELAY_TIMES);
    GPIO_SetBits(I2C_PORT,PIN_SCL);
    Delay_us(DELAY_TIMES);
    GPIO_SetBits(I2C_PORT,PIN_SDA);
    Delay_us(DELAY_TIMES);
}

uint8_t ch57x_soft_i2c_wait_ack(void) {
    uint8_t ucErrTime=0;
    SDA_IN();
    GPIO_SetBits(I2C_PORT,PIN_SDA);
    Delay_us(DELAY_TIMES);
    GPIO_SetBits(I2C_PORT,PIN_SCL);
    Delay_us(DELAY_TIMES);
    while(I2C_READ_SDA()) {
        ucErrTime++;
        if(ucErrTime>250) {
            ch57x_soft_i2c_stop();
            return 1;
        }
    }
    GPIO_ResetBits(I2C_PORT, PIN_SCL);
    Delay_us(DELAY_TIMES);
    return 0;
}

void ch57x_soft_i2c_ack(void) {
    SDA_OUT();
    GPIO_ResetBits(I2C_PORT,PIN_SDA);
    Delay_us(DELAY_TIMES);
    GPIO_SetBits(I2C_PORT,PIN_SCL);
    Delay_us(DELAY_TIMES);
    GPIO_ResetBits(I2C_PORT,PIN_SCL);
}

void ch57x_soft_i2c_nack(void) {
    GPIO_ResetBits(I2C_PORT,PIN_SCL);
    SDA_OUT();
    GPIO_SetBits(I2C_PORT,PIN_SDA);
    Delay_us(DELAY_TIMES);
    GPIO_SetBits(I2C_PORT,PIN_SCL);
    Delay_us(DELAY_TIMES);
    GPIO_ResetBits(I2C_PORT,PIN_SCL);
}

void ch57x_soft_i2c_send_byte(uint8_t txd) {
    uint8_t t;
    SDA_OUT();
    GPIO_ResetBits(I2C_PORT,PIN_SCL);
    for(t=0; t<8; t++) {
        if((txd&0x80)==0x80) {
            GPIO_SetBits(I2C_PORT, PIN_SDA);   //SDA=1
        } else {
            GPIO_ResetBits(I2C_PORT, PIN_SDA);   //SDA=0
        }
        txd<<=1;
        Delay_us(DELAY_TIMES);
        GPIO_SetBits(I2C_PORT,PIN_SCL);
        Delay_us(DELAY_TIMES);
        GPIO_ResetBits(I2C_PORT,PIN_SCL);
        Delay_us(DELAY_TIMES);
    }
    GPIO_SetBits(I2C_PORT, PIN_SDA);
}

uint8_t ch57x_soft_i2c_read_byte(unsigned char ack) {
    unsigned char i,receive=0;
    SDA_IN();
    GPIO_SetBits(I2C_PORT,PIN_SDA);
    for(i=0; i<8; i++ ) {
        GPIO_ResetBits(I2C_PORT,PIN_SCL);
        Delay_us(DELAY_TIMES);
        GPIO_SetBits(I2C_PORT,PIN_SCL);
        Delay_us(DELAY_TIMES);
        receive<<=1;
        if(I2C_READ_SDA()) {
            receive++;
        }
        Delay_us(DELAY_TIMES);
    }
    GPIO_ResetBits(I2C_PORT,PIN_SCL);
    if (!ack)
        ch57x_soft_i2c_nack();//nACK
    else
        ch57x_soft_i2c_ack(); //ACK
    return receive;
}

void ch57x_soft_i2c_single_write(uint8_t daddr,uint8_t addr,uint8_t data) {
    ch57x_soft_i2c_start();
    ch57x_soft_i2c_send_byte(daddr);
    ch57x_soft_i2c_wait_ack();
    ch57x_soft_i2c_send_byte(addr);
    ch57x_soft_i2c_wait_ack();
    ch57x_soft_i2c_send_byte(data);
    ch57x_soft_i2c_wait_ack();
    ch57x_soft_i2c_stop();
}

uint8_t ch57x_soft_i2c_single_read(uint8_t daddr,uint8_t addr) {
    uint8_t data;
    ch57x_soft_i2c_start();
    ch57x_soft_i2c_send_byte(daddr);
    ch57x_soft_i2c_wait_ack();
    ch57x_soft_i2c_send_byte(addr);
    ch57x_soft_i2c_wait_ack();
    ch57x_soft_i2c_start();
    ch57x_soft_i2c_send_byte(daddr+1);
    ch57x_soft_i2c_wait_ack();
    data=ch57x_soft_i2c_read_byte(0);//no ack
    ch57x_soft_i2c_stop();
    return data;
}

//ʼź
void I2C_Start(void)
{
  OLED_SDIN_Set();
  OLED_SCLK_Set();
  OLED_SDIN_Clr();
  OLED_SCLK_Clr();
}

//ź
void I2C_Stop(void)
{
  OLED_SCLK_Set();
  OLED_SDIN_Clr();
  OLED_SDIN_Set();
}

//ȴźӦ
void I2C_WaitAck(void) //źŵĵƽ
{
  OLED_SCLK_Set();
  OLED_SCLK_Clr();
}

void Send_Byte(uint8_t dat)
{
  uint8_t i;
  for(i=0;i<8;i++)
  {
    OLED_SCLK_Clr();//ʱźΪ͵ƽ
    if(dat&0x80)//dat8λλд
    {
      OLED_SDIN_Set();
    }
    else
    {
      OLED_SDIN_Clr();
    }
    OLED_SCLK_Set();//ʱźΪߵƽ
    OLED_SCLK_Clr();//ʱźΪ͵ƽ
    dat<<=1;
  }
}


